//
//  TWSStream.h
//  Twilio Sync Client
//
//  Copyright (c) 2017 Twilio, Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "TWSConstants.h"
#import "TWSStreamMessage.h"

@protocol TWSStreamDelegate;
@class TWSError;

/** Interface for Sync Pub-sub messaging primitive. 
 
 Message Stream is a Sync primitive for real-time pub-sub messaging. 
 Stream Messages are not persisted, they exist only in transit, and 
 will be dropped if (due to congestion or network anomalies) they 
 cannot be delivered promptly.
 */
@interface TWSStream : NSObject

/** An immutable system-assigned identifier of this Message Stream.
 
 @return A string containing Message Stream SID.
 */
- (nonnull NSString *)sid;

/** An optional unique name for this stream, assigned at creation time.
 
 @return The unique name.
 */
- (nullable NSString *)uniqueName;

/** Publish a new message to this message stream.
 
 @param data Contains the payload of the dispatched message. Maximum size in serialized JSON: 4KB.
 @param flowId Specifies the flow identifier associated with this message publication.
 @param completion Completion block that will specify the result of the operation and the unique message identifier if the operation is successful.
 */
- (void)publishMessageWithData:(nonnull TWSData *)data
                        flowId:(NSUInteger)flowId
                    completion:(nullable TWSStreamMessageCompletion)completion;

/** Remove the stream from the system, deleting it.
 
 @param flowId A developer specified identifier for this remote request.
 @param completion Completion block that will specify the result of the operation.
 */
- (void)removeStreamWithFlowId:(NSUInteger)flowId
                    completion:(nullable TWSCompletion)completion;

@end

/** The delegate delivers events that are fired by the Message Stream as a result of asynchronous completion of locally invoked methods, or any remote operations triggered by other actors on the same Stream. */
@protocol TWSStreamDelegate<NSObject>
@optional
/** Called when the message stream is opened or created.
 
 @param stream The reference to the message stream.
 */
- (void)onStreamResultOpened:(nonnull TWSStream *)stream;

/** Called when the stream is removed as a result of a local operation.
 
 @param stream The reference to the message stream.
 @param flowId A developer specified identifier for this remote request.
 */
- (void)onStream:(nonnull TWSStream *)stream resultRemovedForFlowID:(NSUInteger)flowId;

/** Called when the stream message is published as a result of a local operation.
 
 @param stream The reference to the message stream.
 @param messageSID The SID of the published message.
 @param flowId A developer specified identifier for this remote request.
 */
- (void)onStream:(nonnull TWSStream *)stream resultMessagePublished:(nonnull NSString *)messageSID forFlowID:(NSUInteger)flowId;

/** Called when an error occurs as a result of a local operation.

 This event is fired whenever any Message Stream errors occur, typically resulting
 from a locally requested operation.  Notable error conditions include stream being
 non-existent, inaccessible or message publications rate being too high.
 
 @param stream The reference to the message stream.
 @param error The error encounted.  The error object will include the error status code, extended error code and and error message.
 @param flowId A developer specified identifier for this remote request.
 */
- (void)onStream:(nonnull TWSStream *)stream resultErrorOccurred:(nonnull TWSError *)error forFlowID:(NSUInteger)flowId;

/** Called when a message is published to the stream as a result of a remote operation.
 
 @param stream The reference to the message stream.
 @param streamMessage The stream message.
 */
- (void)onStream:(nonnull TWSStream *)stream remoteMessagePublished:(nonnull TWSStreamMessage *)streamMessage;

/** Called when the stream is removed as a result of a remote operation.
 
 @param stream The reference to the removed message stream.
 */
- (void)onStreamRemoteRemoved:(nonnull TWSStream *)stream;

@end
